"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.canWrite = exports.authPassword = exports.authToken = void 0;
const tslib_1 = require("tslib");
const http_status_codes_1 = require("http-status-codes");
const models_1 = require("../models");
const config_1 = (0, tslib_1.__importDefault)(require("../../config"));
const pino_1 = (0, tslib_1.__importDefault)(require("pino"));
const logger = (0, pino_1.default)({ name: "auth.ts", level: config_1.default.logLevel });
/**
 * Middleware to authenticate user based on Basic or Bearer token.
 *
 */
const authToken = () => async (req, res, next) => {
    const { authorization } = req.headers;
    logger.debug({ headers: req.headers, body: req.body });
    if (!authorization) {
        return res.status(http_status_codes_1.StatusCodes.UNAUTHORIZED).send("Missing credentials");
    }
    const [authType, token] = authorization.split(" ");
    if ((authType !== "Basic" && authType !== "Bearer") || !token) {
        return res
            .status(http_status_codes_1.StatusCodes.UNAUTHORIZED)
            .send("Missing authorization token");
    }
    if (authType === "Bearer") {
        const tokenHash = models_1.Token.hashToken(token);
        const tokenInstance = await models_1.Token.findOne({
            where: { token: tokenHash },
        });
        if (!tokenInstance) {
            return res.status(http_status_codes_1.StatusCodes.UNAUTHORIZED).send("Invalid token");
        }
        if (!tokenInstance.checkCIDR(req.ip)) {
            return res.status(http_status_codes_1.StatusCodes.UNAUTHORIZED).send("Invalid IP");
        }
        const user = await models_1.User.findOne({
            where: { _id: tokenInstance.getDataValue("userId") },
        });
        if (!user) {
            return res
                .status(http_status_codes_1.StatusCodes.UNAUTHORIZED)
                .send("Invalid user for token");
        }
        res.locals.user = user;
        res.locals.auth = {
            access: tokenInstance.getDataValue("access"),
        };
    }
    else if (authType === "Basic") {
        const [username, password] = Buffer.from(token, "base64")
            .toString()
            .split(":");
        const user = await models_1.User.findOne({
            where: { name: username },
        });
        if (!user || !user.checkPassword(password)) {
            logger.info({ user }, "did not match password");
            return res
                .status(http_status_codes_1.StatusCodes.UNAUTHORIZED)
                .send("Invalid user or password");
        }
        res.locals.user = user;
    }
    next();
};
exports.authToken = authToken;
const authPassword = () => async (req, res, next) => {
    const { password } = req.body;
    const user = res.locals.user;
    if (!password || !user.checkPassword(password)) {
        return res.status(http_status_codes_1.StatusCodes.UNAUTHORIZED).send("Invalid password");
    }
    next();
};
exports.authPassword = authPassword;
const canWrite = () => async (req, res, next) => {
    var _a, _b;
    if ((_b = (_a = res.locals.auth) === null || _a === void 0 ? void 0 : _a.access) === null || _b === void 0 ? void 0 : _b.includes("readonly")) {
        return res.status(http_status_codes_1.StatusCodes.FORBIDDEN).send("Requires write access");
    }
    next();
};
exports.canWrite = canWrite;
//# sourceMappingURL=auth.js.map