"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Package = void 0;
const sequelize_1 = require("sequelize");
const db_1 = require("./db");
const version_1 = require("./version");
const dist_tag_1 = require("./dist-tag");
const user_1 = require("./user");
class Package extends sequelize_1.Model {
    static async getPackage(packageId) {
        const pkg = await Package.findOne({
            where: { _id: packageId },
            include: [
                { model: version_1.Version, as: "versions" },
                { model: dist_tag_1.DistTag, as: "dist-tags" },
            ],
        });
        if (!pkg) {
            return;
        }
        const versions = pkg.versions.reduce((prev, version) => {
            prev[version.version] = version;
            return prev;
        }, {});
        const distTags = pkg
            .getDataValue("dist-tags")
            .reduce((prev, tag) => {
            prev[tag.name] = tag.version;
            return prev;
        }, {});
        return Object.assign(Object.assign({}, pkg.toJSON()), { versions, "dist-tags": distTags });
    }
    static async addPackage(packageId, userId, name, access) {
        const transaction = await db_1.db.transaction();
        try {
            let pkg = await Package.findOne({
                where: {
                    _id: packageId,
                    userId,
                },
                transaction,
            });
            if (!pkg) {
                pkg = await Package.create({
                    _id: packageId,
                    userId,
                    name,
                    access,
                }, { transaction });
            }
            await transaction.commit();
            return pkg;
        }
        catch (err) {
            await transaction.rollback();
            throw err;
        }
    }
    get versions() {
        return this.getDataValue("versions");
    }
    set versions(versions) {
        this.setDataValue("versions", versions);
    }
}
exports.Package = Package;
Package.init({
    _id: {
        type: sequelize_1.DataTypes.STRING,
        allowNull: false,
        primaryKey: true,
    },
    name: sequelize_1.DataTypes.STRING,
    access: sequelize_1.DataTypes.ENUM("public", "restricted"),
}, {
    sequelize: db_1.db,
    modelName: "Package",
});
Package.hasMany(dist_tag_1.DistTag, { foreignKey: "packageId", as: "dist-tags" });
Package.hasMany(version_1.Version, { foreignKey: "packageId", as: "versions" });
Package.belongsTo(user_1.User, { foreignKey: "userId" });
//# sourceMappingURL=package.js.map