"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Team = void 0;
const sequelize_1 = require("sequelize");
const db_1 = require("./db");
const user_1 = require("./user");
const organization_1 = require("./organization");
const package_1 = require("./package");
const UserTeam = db_1.db.define("UserTeam", {}, { timestamps: false });
const TeamPackage = db_1.db.define("TeamPackage", {
    permissions: {
        type: sequelize_1.DataTypes.ENUM("read-only", "read-write"),
    },
}, { timestamps: false });
class Team extends sequelize_1.Model {
    static async createTeam(scope, name, ownerId, isRoot, description) {
        const transaction = await db_1.db.transaction();
        try {
            let organization = await organization_1.Organization.findOne({
                where: {
                    name: scope,
                },
                transaction,
            });
            if (!organization) {
                if (!isRoot) {
                    throw new Error("Only roots can create organizations");
                }
                organization = await organization_1.Organization.create({
                    name: scope,
                }, { transaction });
                // Add owner as member of the organization
                await organization_1.UserOrganization.create({
                    organizationId: scope,
                    userId: ownerId,
                    role: "owner",
                }, { transaction });
            }
            const team = await Team.create({
                id: `${scope}:${name}`,
                name,
                description,
                organizationId: scope,
            }, {
                transaction,
            });
            await transaction.commit();
            return team;
        }
        catch (err) {
            await transaction.rollback();
            throw err;
        }
    }
    static async removeTeam(scope, team) {
        await Team.destroy({
            where: {
                id: `${scope}:${team}`,
            },
        });
    }
    static async getMembers(scope, teamName) {
        const teamId = `${scope}:${teamName}`;
        return Team.findOne({
            where: { id: teamId },
            include: { model: user_1.User, as: "members" },
        });
    }
    static async addUser(scope, teamName, userName) {
        const transaction = await db_1.db.transaction();
        try {
            const user = await user_1.User.findOne({
                where: { name: userName },
                transaction,
            });
            if (!user) {
                throw new Error("Missing user");
            }
            const teamId = `${scope}:${teamName}`;
            const team = await Team.findOne({ where: { id: teamId }, transaction });
            if (!team) {
                throw new Error("Missing team");
            }
            await UserTeam.create({
                teamId,
                userId: user.getDataValue("_id"),
            }, { transaction });
            await transaction.commit();
        }
        catch (err) {
            await transaction.rollback();
            throw err;
        }
    }
    static async removeUser(scope, teamName, userName) {
        const user = await user_1.User.findOne({ where: { name: userName } });
        if (!user) {
            throw new Error("Missing user");
        }
        const teamId = `${scope}:${teamName}`;
        return UserTeam.destroy({
            where: {
                teamId,
                userId: user.getDataValue("_id"),
            },
        });
    }
    static async grantAccess(scope, teamName, packageId, permissions) {
        const transaction = await db_1.db.transaction();
        try {
            const teamId = `${scope}:${teamName}`;
            const team = await Team.findOne({ where: { id: teamId }, transaction });
            if (!team) {
                throw new Error("Missing team");
            }
            const pkg = await package_1.Package.findOne({
                where: { _id: packageId },
                transaction,
            });
            if (!pkg) {
                throw new Error("Missing package");
            }
            await TeamPackage.create({
                teamId,
                packageId,
                permissions,
                // Denormalize organization for faster queries
                organizationId: team.getDataValue("organizationId"),
            }, { transaction });
            await transaction.commit();
        }
        catch (err) {
            await transaction.rollback();
            throw err;
        }
    }
    static async revokeAccess(scope, teamName, packageId) {
        const teamId = `${scope}:${teamName}`;
        await TeamPackage.destroy({
            where: {
                teamId,
                packageId,
            },
        });
    }
    static async listPackages(scope, teamName) {
        const teamId = `${scope}:${teamName}`;
        return TeamPackage.findAll({
            where: {
                teamId,
            },
        });
    }
    static async listOrganizationPackages(scope) {
        return TeamPackage.findAll({
            where: {
                organizationId: scope,
            },
            attributes: ["packageId", "permissions", ["teamId", "team"]],
        });
    }
    static async checkPermissions(userId, packageId, permissions) {
        // Check which teams have permissions for this package
        // and where given user is member of said teams.
        const teams = await Team.findAll({
            include: [
                { model: user_1.User, as: "members", where: { _id: userId }, attributes: [] },
                {
                    model: package_1.Package,
                    as: "packages",
                    where: { _id: packageId },
                    through: {
                        attributes: ["permissions"],
                        where: {
                            permissions: {
                                [sequelize_1.Op.or]: permissions,
                            },
                        },
                    },
                    attributes: ["_id"],
                },
            ],
            attributes: [],
        });
        return teams.length > 0;
    }
}
exports.Team = Team;
Team.init({
    id: {
        type: sequelize_1.DataTypes.STRING,
        primaryKey: true,
    },
    name: sequelize_1.DataTypes.STRING,
    description: sequelize_1.DataTypes.STRING,
}, {
    sequelize: db_1.db,
    modelName: "Team",
});
Team.belongsTo(organization_1.Organization, { foreignKey: "organizationId" });
// Team membership
user_1.User.belongsToMany(Team, {
    through: UserTeam,
    foreignKey: "userId",
    as: "members",
});
Team.belongsToMany(user_1.User, {
    through: UserTeam,
    foreignKey: "teamId",
    as: "members",
});
// Package permissions
TeamPackage.belongsTo(organization_1.Organization, { foreignKey: "organizationId" });
package_1.Package.belongsToMany(Team, {
    through: TeamPackage,
    foreignKey: "packageId",
    as: "packages",
});
Team.belongsToMany(package_1.Package, {
    through: TeamPackage,
    foreignKey: "teamId",
    as: "packages",
});
//# sourceMappingURL=team.js.map