"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Token = void 0;
const tslib_1 = require("tslib");
const crypto_1 = require("crypto");
const sequelize_1 = require("sequelize");
const uuid = (0, tslib_1.__importStar)(require("uuid"));
const range_check = (0, tslib_1.__importStar)(require("range_check"));
const db_1 = require("./db");
const user_1 = require("./user");
class Token extends sequelize_1.Model {
    static hashToken(token) {
        return (0, crypto_1.createHash)("sha512").update(token).digest("hex");
    }
    static async createTokenForUser(userId, { readonly, cidrWhitelist }) {
        const tokenUuid = uuid.v4();
        const hidden = `${tokenUuid.slice(0, 8)}.....${tokenUuid.slice(-4)}`;
        const token = await Token.create({
            token: tokenUuid,
            hidden,
            access: readonly ? ["readonly"] : ["publish"],
            cidrWhitelist,
            userId,
        });
        return [tokenUuid, token];
    }
    // TODO: implement pagination
    static async getTokens(userId) {
        const tokens = await Token.findAll({
            where: { userId },
            attributes: {
                include: ["key", "createdAt", "access", "cidrWhitelist"],
            },
            order: [["createdAt", "DESC"]],
        });
        const result = {
            objects: tokens.map((token) => token.getPublicData()),
            urls: {},
        };
        return result;
    }
    /**
     * Revokes a token. Only owner or root user can revoke other
     * users tokens.
     *
     * @param userId
     * @param tokenOrKey
     * @param isRoot
     */
    static async revokeToken(userId, tokenOrKey, isRoot) {
        const hashedToken = Token.hashToken(tokenOrKey);
        const where = {
            [sequelize_1.Op.or]: [
                {
                    key: tokenOrKey,
                },
                { token: hashedToken },
            ],
        };
        if (!isRoot) {
            where.userId = userId;
        }
        const numDeleted = await Token.destroy({
            where,
        });
        return numDeleted > 0;
    }
    get cidrWhitelist() {
        return this.getDataValue("cidrWhitelist") || [];
    }
    checkCIDR(ip) {
        var _a;
        // We strip the IPv6 part here for now
        ip = ip.replace("::ffff:", "");
        return ((_a = this.cidrWhitelist) === null || _a === void 0 ? void 0 : _a.length)
            ? range_check.inRange(ip, this.cidrWhitelist)
            : true;
    }
    getPublicData() {
        return {
            key: this.getDataValue("key"),
            token: this.getDataValue("hidden"),
            created: this.getDataValue("createdAt"),
            cidr_whitelist: this.cidrWhitelist,
            readonly: this.getDataValue("access").includes("readonly"),
        };
    }
}
exports.Token = Token;
Token.init({
    key: {
        type: sequelize_1.DataTypes.UUID,
        defaultValue: sequelize_1.DataTypes.UUIDV4,
        primaryKey: true,
    },
    token: {
        type: sequelize_1.DataTypes.STRING,
        set(token) {
            if (token) {
                this.setDataValue("token", Token.hashToken(token));
            }
        },
    },
    hidden: sequelize_1.DataTypes.STRING,
    cidrWhitelist: {
        type: sequelize_1.DataTypes.STRING,
        set(cidrs) {
            cidrs && this.setDataValue("cidrWhitelist", cidrs.join(","));
        },
        get() {
            const cidr = this.getDataValue("cidrWhitelist");
            return (cidr && cidr.split(",")) || void 0;
        },
    },
    name: sequelize_1.DataTypes.STRING,
    lastUsed: sequelize_1.DataTypes.DATE,
    useCount: sequelize_1.DataTypes.INTEGER,
    access: {
        type: sequelize_1.DataTypes.STRING,
        get() {
            const access = this.getDataValue("access");
            return (access && access.split(",")) || void 0;
        },
        set(access) {
            access && this.setDataValue("access", access.join(","));
        },
    },
}, {
    sequelize: db_1.db,
    modelName: "Token",
});
Token.belongsTo(user_1.User, { foreignKey: "userId" });
/*
export class Token {
  ownerId: string;
  key: string;
  name: string;
  token: string; // Encrypted token
  lastUsed: Date;
  createdAt: Date;
}
*/
//# sourceMappingURL=token.js.map