"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.router = void 0;
/**
 *
 */
const http_status_codes_1 = require("http-status-codes");
const express_1 = require("express");
const asyncWrap_1 = require("../middleware/asyncWrap");
const team_1 = require("../models/team");
const organization_1 = require("../models/organization");
const user_1 = require("../models/user");
const enums_1 = require("../enums");
const middleware_1 = require("../middleware");
exports.router = (0, express_1.Router)();
exports.router.get("/-/org/:scope/user", (0, middleware_1.canAccessOrganization)(enums_1.OrganizationAction.listMembers), (0, asyncWrap_1.asyncWrap)(async (req, res) => {
    const { scope } = req.params;
    const members = await organization_1.UserOrganization.findAll({
        where: {
            organizationId: scope,
        },
        attributes: ["userId", "role"],
    });
    res.status(http_status_codes_1.StatusCodes.CREATED).json(members.reduce((prev, member) => {
        const [_, username] = member.getDataValue("userId").split(":");
        prev[username] = member.getDataValue("role");
        return prev;
    }, {}));
}));
exports.router.put("/-/org/:scope/user", (0, express_1.json)(), (0, middleware_1.canWrite)(), (0, middleware_1.canAccessOrganization)(enums_1.OrganizationAction.addMember, enums_1.OrganizationAction.changeMemberRole), (0, asyncWrap_1.asyncWrap)(async (req, res) => {
    const { scope } = req.params;
    const { user: username, role } = req.body;
    const user = await user_1.User.findOne({
        where: {
            name: username,
        },
    });
    if (!user) {
        res.status(http_status_codes_1.StatusCodes.NOT_FOUND).end("User not found");
    }
    await organization_1.UserOrganization.upsert({
        organizationId: scope,
        userId: user.getDataValue("_id"),
        role,
    });
    res.status(http_status_codes_1.StatusCodes.OK).json({
        org: {
            name: scope,
            size: await organization_1.UserOrganization.count({
                where: {
                    organizationId: scope,
                },
            }),
        },
        user: username,
        role,
    });
}));
exports.router.delete("/-/org/:scope/user", (0, express_1.json)(), (0, middleware_1.canWrite)(), (0, middleware_1.canAccessOrganization)(enums_1.OrganizationAction.removeMember), (0, asyncWrap_1.asyncWrap)(async (req, res) => {
    const { scope } = req.params;
    const { user: username } = req.body;
    const user = await user_1.User.findOne({
        where: {
            name: username,
        },
    });
    if (!user) {
        res.status(http_status_codes_1.StatusCodes.NOT_FOUND).end("User not found");
    }
    await organization_1.UserOrganization.destroy({
        where: {
            organizationId: scope,
            userId: user.getDataValue("_id"),
        },
    });
    res.status(http_status_codes_1.StatusCodes.OK).end();
}));
/**
 * Create a new team on a given organization.
 *
 * Note: An organization will created automatically if not existing yet,
 * however, only "root" users have the right to create organizations.
 *
 * Note: Only organization owner, admins or root users are able to add teams
 * to any organization.
 */
exports.router.put("/-/org/:scope/team", (0, express_1.json)(), (0, middleware_1.canWrite)(), (0, middleware_1.canAccessOrganization)(enums_1.OrganizationAction.createTeam), (0, asyncWrap_1.asyncWrap)(async (req, res) => {
    const { scope } = req.params;
    const { name, description } = req.body;
    const { _id: ownerId, type } = res.locals.user;
    const isRoot = type === "root";
    try {
        const team = await team_1.Team.createTeam(scope, name, ownerId, isRoot, description);
        res.status(http_status_codes_1.StatusCodes.CREATED).json(team);
    }
    catch (err) {
        res.status(http_status_codes_1.StatusCodes.FORBIDDEN).send(err);
    }
}));
/**
 * Get teams belonging to an Organization
 *
 */
exports.router.get("/-/org/:scope/team", (0, middleware_1.canAccessOrganization)(enums_1.OrganizationAction.listTeams), (0, asyncWrap_1.asyncWrap)(async (req, res) => {
    const { scope } = req.params;
    const teams = await team_1.Team.findAll({
        where: {
            organizationId: scope,
        },
    });
    res
        .status(http_status_codes_1.StatusCodes.CREATED)
        .json(teams.map((team) => team.getDataValue("name")));
}));
/**
 * List all packages that this organization has access to.
 */
exports.router.get("/-/org/:scope/package", (0, middleware_1.canAccessOrganization)(enums_1.OrganizationAction.manageTeamPackageAccess), (0, asyncWrap_1.asyncWrap)(async (req, res) => {
    const { scope } = req.params;
    // We assume all packages at least are part of one team (as developers)
    const packages = await team_1.Team.listOrganizationPackages(scope);
    res.status(http_status_codes_1.StatusCodes.OK).json(packages);
}));
/**
 * List all packages that this user as organization has access to.
 */
/*
router.get(
  "/-/org/:user/package",
  json(),
  authToken([]),
  asyncWrap(async (req: Request, res: Response) => {
    const { user } = req.params;

    // We could list all the packages the user "owns", as well as all the packages
    // the teams the user is part of have access to.

    res.status(StatusCodes.NOT_ACCEPTABLE).end();
  })
);
*/
//# sourceMappingURL=organizations.js.map