"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.router = void 0;
const tslib_1 = require("tslib");
/**
 *
 */
const http_status_codes_1 = require("http-status-codes");
const express_1 = require("express");
const pino_1 = (0, tslib_1.__importDefault)(require("pino"));
const middleware_1 = require("../middleware");
const storage_1 = require("../storage");
const package_1 = require("../models/package");
const version_1 = require("../models/version");
const team_1 = require("../models/team");
const organization_1 = require("../models/organization");
const enums_1 = require("../enums");
const utils_1 = require("../utils");
const config_1 = (0, tslib_1.__importDefault)(require("../../config"));
const logger = (0, pino_1.default)();
const storage = (0, storage_1.getStorage)();
exports.router = (0, express_1.Router)();
async function getPackageMeta(res, packageId) {
    const pkg = await package_1.Package.getPackage(packageId);
    if (!pkg) {
        return res.status(http_status_codes_1.StatusCodes.NOT_FOUND).end();
    }
    res.json(pkg);
}
/**
 * Get package metadata.
 *
 */
exports.router.get("/:packageId", (0, middleware_1.asyncWrap)(async (req, res) => {
    const { packageId } = req.params;
    if (!(0, utils_1.isScoped)(packageId) ||
        (await team_1.Team.checkPermissions(res.locals.user._id, packageId, [
            "read-only",
            "read-write",
        ]))) {
        await getPackageMeta(res, packageId);
    }
    else {
        res.status(http_status_codes_1.StatusCodes.FORBIDDEN).end();
    }
}));
/**
 * Get a scoped package metadata.
 * Note: This endpoint does not seem to be needed
 */
exports.router.get("/@:scope/:name", (0, middleware_1.asyncWrap)(async (req, res) => {
    const packageId = `@${req.params.scope}/${req.params.name}`;
    res.status(http_status_codes_1.StatusCodes.FORBIDDEN).end();
}));
/**
 * Get a public package version file
 *
 * Example: https://registry.npmjs.org/uuid/-/uuid-0.0.1.tgz"
 *
 */
exports.router.get("/:package/-/:pkgFile", (0, middleware_1.asyncWrap)(async (req, res) => {
    (0, storage_1.downloadPackage)(res, storage, req.params.pkgFile);
}));
/*
 * Download a binary tarball for the given version.
 * Example:  https://registry.npmjs.org/koa-bodyparser/-/koa-bodyparser-0.0.1.tgz
 */
exports.router.get("/@:scope/:package/-/@:scope2/:version", (0, middleware_1.asyncWrap)(async (req, res) => {
    const { scope, package: pkgName, version } = req.params;
    const packageId = `@${scope}/${pkgName}`;
    if (await team_1.Team.checkPermissions(res.locals.user._id, packageId, [
        "read-only",
        "read-write",
    ])) {
        (0, storage_1.downloadPackage)(res, storage, `@${scope}/${version}`);
    }
    else {
        res.status(http_status_codes_1.StatusCodes.FORBIDDEN).end();
    }
}));
exports.router.put("/:package", (0, express_1.json)({ limit: config_1.default.storage.maxPackageSize }), (0, middleware_1.canWrite)(), (0, middleware_1.asyncWrap)(async (req, res) => {
    logger.debug({ path: req.path }, "Publish");
    const { _id, name, versions, _attachments, access = (0, utils_1.isScoped)(_id) ? "restricted" : "public", } = req.body;
    const { _id: userId } = res.locals.user;
    if (!(0, utils_1.isValidPackageName)(name)) {
        return res.status(http_status_codes_1.StatusCodes.NOT_ACCEPTABLE).end("Invalid package name");
    }
    try {
        let canAccess = false;
        const scope = (0, utils_1.isScoped)(_id) ? _id.split("/")[0].substr(1) : void 0;
        if (scope) {
            const role = await organization_1.Organization.getMemberRole(scope, userId);
            if (role) {
                if (await organization_1.Organization.checkPermissions(role, enums_1.OrganizationAction.publishPackage)) {
                    canAccess = true;
                }
            }
        }
        if (!canAccess &&
            (await team_1.Team.checkPermissions(userId, _id, ["read-write"]))) {
            canAccess = true;
        }
        if (!canAccess) {
            return res.status(http_status_codes_1.StatusCodes.FORBIDDEN).end();
        }
        const pkg = await package_1.Package.addPackage(_id, userId, name, access);
        const port = req.socket.localPort;
        const tarballPrefix = `${req.protocol}://${req.hostname}:${port}/${_id}/-/`;
        const distTags = Object.entries(req.body["dist-tags"]);
        const [[tagName]] = distTags;
        const result = await version_1.Version.addVersions(storage, _id, versions, _attachments, tarballPrefix, tagName);
        res.status(http_status_codes_1.StatusCodes.OK).end();
    }
    catch (err) {
        logger.error(err, "Error adding package or versions");
        if (err.code === "EEXIST" ||
            err.type === "UniqueConstraintError") {
            res.status(http_status_codes_1.StatusCodes.UNPROCESSABLE_ENTITY).end();
        }
        else {
            throw err;
        }
    }
}));
/**
 * Get all the users that have access to the given package
 */
exports.router.get("/-/package/@:scope/:name/collaborators", (0, middleware_1.asyncWrap)(async (req, res) => {
    res.status(http_status_codes_1.StatusCodes.NOT_ACCEPTABLE).end();
}));
exports.router.get("/-/package/:name/collaborators", (0, middleware_1.asyncWrap)(async (req, res) => {
    res.status(http_status_codes_1.StatusCodes.NOT_ACCEPTABLE).end();
}));
//# sourceMappingURL=packages.js.map